/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package eb;

import java.rmi.RemoteException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Enumeration;
import java.util.Properties;
import java.util.Vector;
import javax.ejb.EntityBean;
import javax.ejb.EntityContext;
import javax.ejb.FinderException;
import javax.ejb.ObjectNotFoundException;
import javax.ejb.RemoveException;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.sql.DataSource;


/**
 * AccountExplBean is an entity bean with "bean managed persistence". 
 * The state of an instance is stored into a relational database. 
 * The following table should exist:
 * create table ACCOUNT (ACCNO    integer primary key,
 *                       CUSTOMER varchar(30),
 *                       BALANCE  number(15,4));
 *
 * Note : In order to keep the code readable, not all database errors have
 * been handled. In particular the closing of the statement and of the
 * connexion would have being put in a "finally" block. 
 * 
 */

public class AccountExplBean implements EntityBean {

    // Static info for Database
    // (The DataSource can be shared among all instances)
    protected static DataSource dataSource = null;

    // Keep the reference on the EntityContext
    protected EntityContext entityContext;

    // Object state - must de public (EJB spec.)
    public int accno;
    public String customer;
    public double balance;

    /*========================= ejbCreate methods ============================*/

    /**
     * There must be one ejbCreate() method per create() method on the Home interface, 
     *  and with the same signature.
     *
     * @param accno account number
     * @param customer customer name
     * @param balance initial balance
     *
     * @return pk primary key
     *
     * @exception RemoteException If the instance could not perform the function 
     *            requested by the container 
     */
    public AccountBeanPK ejbCreate(int val_accno, String val_customer, double val_balance) 
	throws RemoteException {

	// Init object state
	accno = val_accno;
	customer = val_customer;
	balance = val_balance;

	// Create the primary key
	AccountBeanPK pk = new AccountBeanPK(val_accno);

	try {
	    // get a connection for this transaction context
	    Connection conn = dataSource.getConnection();

	    // create object in DB
	    PreparedStatement stmt = conn.prepareStatement("insert into accountsample (accno, customer, balance) values (?, ?, ?)");
	    stmt.setInt(1, pk.accno);
	    stmt.setString(2, customer);
	    stmt.setDouble(3, balance);
	    stmt.executeUpdate();

	    // close statement
	    stmt.close();
	    
	    // release connection
	    conn.close();

	} catch (SQLException e) {
	    throw new java.rmi.RemoteException("Failed to create bean in database", e);
	}

	// Return the primary key
	return pk;
    }
    
    /**
     * Each ejbCreate method should have a matching ejbPostCreate method
     */
    public void ejbPostCreate(int val_accno, String val_customer, double val_balance) 
	throws RemoteException {
	// Nothing to be done for this simple example.
    }



    /*====================== javax.ejb.EntityBean implementation =================*/

    /** 
     * A container invokes this method when the instance is taken out of the pool 
     *  of available instances to become associated with a specific EJB object. 
     * This method transitions the instance to the ready state. 
     * 
     * This method executes in an unspecified transaction context. 
     *
     * @exception RemoteException If the instance could not perform the function 
     *         requested by the container because of an system-level error. 
     */
    public void ejbActivate() throws RemoteException {
	// Nothing to be done for this simple example.
    }

    /**
     * A container invokes this method to instruct the instance to synchronize 
     *  its state by loading it state from the underlying database. 
     * This method always executes in the proper transaction context. 
     * 
     * @exception RemoteException  If the instance could not perform the function 
     *          requested by the container because of a system-level error. 
     *
     */
    public void ejbLoad() throws RemoteException {

	try {
	    // get a connection for this transaction context
	    Connection conn = dataSource.getConnection();

	    // find account in DB
	    PreparedStatement stmt = conn.prepareStatement("select customer,balance from accountsample where accno=?");
	    AccountBeanPK pk = (AccountBeanPK) entityContext.getPrimaryKey();
	    stmt.setInt(1, pk.accno);
	    ResultSet rs = stmt.executeQuery();
            if (rs.next() == false) {
                throw new RemoteException("Failed to load bean from database");
            }

	    // update object state
	    accno = pk.accno;
	    customer = rs.getString("customer");
	    balance  = rs.getDouble("balance");

	    // close statement
	    stmt.close();

	    // release connection
	    conn.close();

	} catch (SQLException e) {	
	    throw new RemoteException("Failed to load bean from database", e);
	} 
    }

    /**
     * A container invokes this method on an instance before the instance becomes 
     *  disassociated with a specific EJB object. After this method completes, the 
     *  container will place the instance into the pool of available instances. 
     * 
     * This method executes in an unspecified transaction context.
     * 
     * @exception RemoteException  If the instance could not perform the function 
     *         requested by the container because of a system-level error. 
     */
    public void ejbPassivate() throws RemoteException {
	// Nothing to be done for this simple example.
    }

    /**
     * A container invokes this method before it removes the EJB object 
     *  that is currently associated with the instance. This method is invoked 
     *  when a client invokes a remove operation on the enterprise Bean's home 
     *  interface or the EJB object's remote interface. This method transitions 
     *  the instance from the ready state to the pool of available instances. 
     *
     * This method is called in the transaction context of the remove operation. 
     *
     * @exception RemoteException  Thrown if the instance could not perform the function 
     *         requested by the container because of a system-level error. 
     * @exception RemoveException  The enterprise Bean does not allow destruction of the object. 
     *
     */
    public void ejbRemove() throws RemoteException, RemoveException {

	try {
	    // get a connection for this transaction context
	    Connection conn = dataSource.getConnection();

	    // delete Object in DB
	    PreparedStatement stmt = conn.prepareStatement("delete from accountsample where accno=?");
	    AccountBeanPK pk = (AccountBeanPK) entityContext.getPrimaryKey();
	    stmt.setInt(1, pk.accno);
	    stmt.executeUpdate();

	    // close statement
	    stmt.close();

	    // release connection
	    conn.close();

	} catch (SQLException e) {
	    throw new java.rmi.RemoteException("Failed to delete bean from database", e);
	}
    }

    /**
     * A container invokes this method to instruct the instance to synchronize 
     *  its state by storing it to the underlying database. 
     * 
     * This method always executes in the proper transaction context. 
     * 
     * @exception: RemoteException  Thrown if the instance could not perform the function 
     *         requested by the container because of a system-level error. 
     */
    public void ejbStore() throws RemoteException {

	try {
	    // get a connection for this transaction context
	    Connection conn = dataSource.getConnection();

	    // store Object state in DB
	    PreparedStatement stmt = conn.prepareStatement("update accountsample set customer=?,balance=? where accno=?");
	    stmt.setString(1, customer);
	    stmt.setDouble(2, balance);
	    AccountBeanPK pk = (AccountBeanPK) entityContext.getPrimaryKey();
	    stmt.setInt(3, pk.accno);
	    stmt.executeUpdate();

	    // close statement
	    stmt.close();

	    // release connection
	    conn.close();

	} catch (SQLException e) {
	    throw new java.rmi.RemoteException("Failed to store bean to database", e);
	}
    }
  

    /**
     * Sets the associated entity context. The container invokes this method 
     *  on an instance after the instance has been created. 
     * 
     * This method is called in an unspecified transaction context. 
     * 
     * @param ctx - An EntityContext interface for the instance. The instance should 
     *              store the reference to the context in an instance variable. 
     * @exception RemoteException  Thrown if the instance could not perform the function
     *        requested by the container because of a system-level error.
     */
    public void setEntityContext(EntityContext ctx) throws RemoteException { 

	// Keep the entity context in object
	entityContext = ctx;

	// First time only: lookup for Datasource in JNDI.
	if (dataSource == null) {
	    Properties env = ctx.getEnvironment();
	    Context initialContext = null;
	    try {
		initialContext = new InitialContext(env); 
	    } catch (Exception e) {
		throw new RemoteException("Cannot get InitialContext ", e);
	    }
	    
	    String dataSourceName = env.getProperty("datasource.name");  
	    try {
		dataSource = (DataSource)initialContext.lookup(dataSourceName);
	    } catch (Exception e) {
		throw new RemoteException("cannot lookup " + dataSourceName, e);
	    }
	}
    }


    /**
     * Unsets the associated entity context. The container calls this method 
     *  before removing the instance. This is the last method that the container 
     *  invokes on the instance. The Java garbage collector will eventually invoke 
     *  the finalize() method on the instance. 
     *
     * This method is called in an unspecified transaction context. 
     * 
     * @exception RemoteException  Thrown if the instance could not perform the function 
     *         requested by the container because of a system-level error.  
     */
    public void unsetEntityContext() {
	entityContext = null;
    }


    /*============================ ejbFind methods ===========================*/

    /**
     * There must be one ejbFind method per find method on the Home interface, 
     *  and with the same signature.
     * ejbFindByPrimaryKey is the only mandatory ejbFind method.
     *
     * @param  pk The primary key
     *
     * @return pk The primary key
     *
     * @exception FinderException - Failed to execute the query.
     * @exception ObjectNotFoundException - Object not found for this primary key.
     */
    public AccountBeanPK ejbFindByPrimaryKey(AccountBeanPK pk) 
	throws ObjectNotFoundException, FinderException {

	try {
	    // get a connection for this transaction context
	    Connection conn = dataSource.getConnection();

	    // lookup for this primary key in DB
	    PreparedStatement stmt = conn.prepareStatement("select accno from accountsample where accno=?");
	    stmt.setInt(1, pk.accno);
	    ResultSet rs = stmt.executeQuery();
	    if (rs.next() == false) {
		throw new javax.ejb.ObjectNotFoundException();
	    }

	    // close statement
	    stmt.close();

	    // release connection
	    conn.close();

	} catch (SQLException e) {
	    throw new javax.ejb.FinderException("Failed to executeQuery " +e);
	}

	// return primary key
	return pk;
    }

    /**
     * Find Account by its account number
     *
     * @return pk The primary key
     *
     * @exception FinderException - Failed to execute the query.
     * @exception ObjectNotFoundException - Object not found for this account number
     */
    public AccountBeanPK ejbFindByNumber(int accno) 
	throws ObjectNotFoundException, FinderException {

	try {
	    // get a connection for this transaction context
	    Connection conn = dataSource.getConnection();

	    // lookup for this primary key in DB
	    PreparedStatement stmt = conn.prepareStatement("select accno from accountsample where accno=?");
	    stmt.setInt(1, accno);
	    ResultSet rs = stmt.executeQuery();
	    if (rs.next() == false) {
		throw new javax.ejb.ObjectNotFoundException();
	    }

	    // close statement
	    stmt.close();

	    // release connection
	    conn.close();

	} catch (SQLException e) {
	    throw new javax.ejb.FinderException("Failed to executeQuery " +e);
	}

	// return a primary key for this account
	AccountBeanPK pk = new AccountBeanPK(accno);
	return pk;
    }

    /**
     * Creates an enumeration of primary keys for all accounts
     *
     * @return pkv The primary keys
     *
     * @exception FinderException - Failed to execute the query.
     */
    public Enumeration ejbFindAllAccounts() 
	throws FinderException
    {
	Vector pkv = new Vector();
	try {
	    // get a connection for this transaction context
	    Connection conn = dataSource.getConnection();

	    // Lookup for all accounts in DB
	    PreparedStatement stmt = conn.prepareStatement("select accno from accountsample");
	    ResultSet rs = stmt.executeQuery();

	    // Build the vector of primary keys
	    while (rs.next()) {
		AccountBeanPK pk = new AccountBeanPK(rs.getInt("accno"));
		pkv.addElement((Object)pk);
	    };

	    // close statement
	    stmt.close();

	    // release connection
	    conn.close();

	} catch (SQLException e) {
	    throw new javax.ejb.FinderException("Failed to executeQuery " +e);
	}

	// return primary keys
	return pkv.elements();
    }


    /*========================= Account implementation ============================*/

    /**
     * Business method for returning the balance.
     *
     * @return balance
     *
     */
    public double getBalance() throws RemoteException {

	return balance;
    }


    /**
     * Business method for updating the balance.
     *
     * @param d balance to update
     * 
     */
    public void setBalance(double d) throws RemoteException {

	balance = balance + d;
    }

  
    /**
     * Business method for returning the customer.
     *
     * @return customer
     *
     */
    public String  getCustomer() throws RemoteException {

	return customer;
    }


    /**
     * Business method for changing the customer name.
     *
     * @param c customer to update
     * 
     */
    public void setCustomer(String c) throws RemoteException {

	customer = c;
    }


    /**
     * Business method to get the Account number
     */
    public int getNumber() throws RemoteException {
	return accno;
    }
} 
