/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package eb;

import java.io.BufferedReader; 
import java.io.FileInputStream;
import java.io.InputStreamReader; 
import java.io.IOException;
import java.rmi.RemoteException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.transaction.UserTransaction;
import javax.naming.Context;
import javax.naming.InitialContext;
import eb.Account;
import eb.AccountHome;

/**
 * Sample for entity beans.
 * Usage: 
 * implicit persistance: java -D<url> eb.ClientAccount AccountImplHome
 * explicit persistance: java -D<url> eb.ClientAccount AccountExplHome
 * <url> is the property used to locate rmiregistry, i.e. something like:
 *      java.naming.provider.url="rmi://hostname:1099"
 *
 */
public class ClientAccount {

    private static UserTransaction utx = null;

    private static void accountList(AccountHome h) {
	Enumeration alist;
	Account acc;
	try {
	    utx.begin();	// faster if made inside a Tx
	    alist = h.findAllAccounts();
	    while (alist.hasMoreElements()) {
		acc = (Account) alist.nextElement();
		System.out.println(acc.getNumber()+" "+acc.getCustomer()+" "+acc.getBalance());
	    }
	    utx.commit();
	} catch (Exception e) {
	    System.err.println("Exception getting account list: " + e);
	    System.exit(2);
	}
    }
   
    public static void main(String[] args) {

	// 1st arg. is the bean home (AccountImplHome or AccountExplHome)
	String beanName = args[0];

	// Set environment properties
	Properties env = System.getProperties();
	// JNDI is above rmiregistry
	env.put("java.naming.factory.initial", "com.sun.jndi.rmi.registry.RegistryContextFactory");
	// java.naming.provider.url can be set with a -D option on java command.
	// you can choose instead to uncomment the following line, set with your own url.
	// env.put("java.naming.provider.url", "rmi://hostname:1099");

	// get JNDI initial context
	Context initialContext = null;
	try {
	    initialContext = new InitialContext(env);
	} catch (Exception e) {
	    System.err.println("Cannot get initial context for JNDI: " + e);
	    System.exit(2);
	}

	// We want to start transactions from client: get UserTransaction
	System.out.println("Getting a UserTransaction object from JNDI");
	try {
	    utx = (UserTransaction) initialContext.lookup("javax.transaction.UserTransaction");
	} catch (Exception e) {
	    System.err.println("Cannot lookup UserTransaction: "+e);
	    System.exit(2);
	}

	// Connecting to Home thru JNDI
	System.out.println("Connecting to the AccountHome");
	AccountHome home = null;
	try {
	    home = (AccountHome) initialContext.lookup(beanName);
	} catch (Exception e) {
	    System.err.println("Cannot lookup "+beanName+": " +e);
	    System.exit(2);
	}

	// List existing Accounts
	System.out.println("Getting the list of existing accounts in database");
	accountList(home);

	// Create a first Account
	System.out.println("Creating a new Account in database");
	Account a1 = null;
	try {
	    a1 = home.create(109, "John Smith", 0);
	} catch (Exception e) {
	    System.err.println("Cannot create Account: " + e);
	    System.exit(2);
	}

	// Find a second Account
	System.out.println("Finding an Account by its number in database");
	Account a2 = null;
	try {
	    a2 = home.findByNumber(102);
	} catch (Exception e) {
	    System.err.println("Cannot find Account: " + e);
	    System.exit(2);
	}

	// First transaction (committed):
	// transfert 100 from a2 to a1
	System.out.println("Starting a first transaction, that will be committed");
	try {
	    double value = 100.00;
	    utx.begin();
	    a1.setBalance(value);
	    a2.setBalance(-value);
	    utx.commit();
	} catch (Exception e) {
	    System.err.println("exception during 1st Tx: " + e);
	    System.exit(2);
	}

	// Start another transaction (rolled back):
	// transfert 20 from a1 to a2
	System.out.println("Starting a second transaction, that will be rolled back");
	try {
	    double value = 20.00;
	    utx.begin();
	    a1.setBalance(-value);
	    a2.setBalance(value);
	    utx.rollback();
	} catch (Exception e) {
	    System.err.println("exception during 2nd Tx: " + e);
	    System.exit(2);
	}

	// List existing Accounts
	System.out.println("Getting the new list of accounts in database");
	accountList(home);

	// Delete account
	System.out.println("Removing Account previously created in database");
	try {
	    a1.remove();
	} catch (Exception e) {
	    System.err.println("exception during remove: " + e);
	    System.exit(2);
	}

	// Exit program
	System.out.println("ClientAccount terminated");
    }
	    
}


