/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.common;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.lang.Throwable;
import java.util.Hashtable;
import java.util.Properties;
import java.util.StringTokenizer;
import java.util.Vector;

import javax.naming.CompoundName;
import javax.naming.Context;
import javax.naming.InitialContext;


/**
 * This class manages global variables and properties used by EJB Server. 
 * These properties are read using the rules defines in Prop.java.
 * A static design pattern is used. 
 */
public class Env {

    static private final int  trace      = Trace.DB_2;

    // Variables related to OS name
    static public  final int  WINDOWS_NT = 0;
    static public  final int  AIX        = 1;
    static public  final int  SOLARIS    = 2;
    static private       int  osName     = -1;

    // Variables related to Java Version
    static public  final int  JAVA_1_1_6  = 0;
    static public  final int  JAVA_1_1_7  = 1;
    static public  final int  JAVA_1_1_8  = 2;
    static public  final int  JAVA_1_2    = 4;
    static private       int  javaVersion = -1;

    // Variables related to server properties
    static private String     jndiSpi        = null;
    static private String     jndiUrl        = null;
    static private String     jndiDir        = null;
    static private String     jndiCase       = null;
    static private Context    initialContext = null;
    static private Properties envJNDI        = null;


    static {
	envJNDI = new Properties();
	String factoryInitial = Prop.svr.getValue("java.naming.factory.initial");
	String providerUrl = Prop.svr.getValue("java.naming.provider.url");
	String syntaxDirection = Prop.svr.getValue("jndi.syntax.direction");
	String syntaxIgnoreCase = Prop.svr.getValue("jndi.syntax.ignorecase");

	envJNDI.put("java.naming.factory.initial", factoryInitial);
	envJNDI.put("java.naming.provider.url", providerUrl);
	envJNDI.put("jndi.syntax.direction", syntaxDirection);
	envJNDI.put("jndi.syntax.ignorecase", syntaxIgnoreCase);
	// initial context
	try {
	    initialContext = new InitialContext(envJNDI);
	} catch (Exception e) {
	    Trace.errln("    new InitialContext() : " + e);
	    e.printStackTrace();
	    System.exit(2);
	}
    }


    /**
     * Gets the initialContext for JNDI. 
     * @return JNDI initial context.
     */
    static public Context getInitialContext() {
	return initialContext;
    }


    /**
     * Gets JNDI syntax. 
     * @return  Syntax properties
     */
    static public Properties getSyntaxJNDI() {
	return envJNDI;
    }


    /**
     * Returns a compound name according to JNDI syntax. No more used 
     * @return	CompoundName or null if error
     */
    static private CompoundName getCompoundName(String name) {

	CompoundName cName = null;
	String       sName = null;

	sName = name;
	try {
	    cName = new CompoundName(sName, getSyntaxJNDI());
	} catch (Exception e) {
	    Trace.errln("Invalid compound name "+sName+" : " + e);
	}
	return cName;
    }


    /**
     * Gets OS.
     * @return	WINDOWS_NT|SOLARIS|AIX or -1 if error
     */
    static public int getOS() {

	if (osName==-1) {
	    // Sets OS name
	    String stros = System.getProperties().getProperty("os.name", "");
	    if (stros.equalsIgnoreCase("Windows NT"))
		osName = WINDOWS_NT;
	    if (stros.equalsIgnoreCase("AIX"))
		osName = AIX;
	    if (stros.equalsIgnoreCase("Solaris"))
		osName = SOLARIS;
	} 
	return osName;
    }


    /**
     * Gets Java Version.
     * @return	JAVA_1_1_6|JAVA_1_1_7 or -1 if error
     */
    static public int getJavaVersion() {

	if (javaVersion==-1) {
	    // Sets Java Version
	    String strjv = System.getProperties().getProperty("java.version", "");
	    if (strjv.indexOf("1.1.6") == 0)
		javaVersion = JAVA_1_1_6;
	    if (strjv.indexOf("1.1.7") == 0)
		javaVersion = JAVA_1_1_7;
	    if (strjv.indexOf("1.1.8") == 0)
		javaVersion = JAVA_1_1_8;
	    if (strjv.indexOf("1.2") == 0)
		javaVersion = JAVA_1_2;
	} 
	return javaVersion;
    }


    /**
     * Gets beans descriptors names vector. 
     * @return Beans descriptors names vector.
     */
    static public Vector getDescBeans() {
	Trace.outln(trace, "String Env.getDescBeans() ");

	Vector retVect   = new Vector();
	String descNames = Prop.svr.getValue("jonas.beans.descriptors", null);
	if (descNames != null) {
	    StringTokenizer st        = new StringTokenizer(descNames, ",");
	    while(st.hasMoreTokens()) // Loops on beans descriptors
		retVect.addElement(st.nextToken().trim());
	    Trace.outln(trace, "    descriptors="+retVect.toString());
	} else retVect = null;
	return retVect;
    }


    /**
     * Gets DataSources names vector. 
     * @return DataSources names vector.
     */
    static public Vector getDataSources() {
	Trace.outln(trace, "String Env.getDataSources() ");

	Vector retVect     = new Vector();
	String dataSources = Prop.svr.getValue("jonas.datasources", null);
	if (dataSources != null) {
	    StringTokenizer st = new StringTokenizer(dataSources, ",");
	    while(st.hasMoreTokens()) // Loops on dataSources
		retVect.addElement(st.nextToken().trim());
	    Trace.outln(trace, "    datasources="+retVect.toString());
	} else retVect = null;
	return retVect;
    }

    /** Gets JOnAS Server name.
     * @return JOnAS Server name.
     */
    static public String getJonasName() {
	String name = Prop.svr.getValue("jonas.name", "");
	Trace.outln(trace, "String Env.getJonasName() -> "+name);
	return(name);
    }

}

