/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.common;

import java.io.*;
import java.lang.String;
import java.util.Enumeration;
import java.util.Vector;
import java.util.StringTokenizer;
import java.util.zip.*;


/**
 * Represents a jar archive file, and implements some tool methods: 
 * <ul>
 * <li>
 * Unpack the jar file.
 * <li>
 * Is the jar file an EJB-jar file ?
 * <li>
 * Give the list of the names of the serialized Deployment Descriptors
 * included in the EJB-jar file.
 * </ul>
 */


public class JarFile {

    private static final String MANIFEST_NAME = new String("META-INF"+File.separatorChar+"MANIFEST.MF");

    // Path of the jar-file
    private String mName;
    // Is the jar file has been unpacked ?
    private boolean mIsUnpacked;
    // Path of the directory where the jar file has been unpacked (set in unpack())
    private String mDir;
    // EJB-Names (i.e. path of the unpacked enterprise Bean's serialized
    // deployment descriptors files) (set in isEJBJarFile())
    private Vector mEJBNames;


    /**
     * Constructor method of the JarFile class
     *
     * @param name path of the jar file
     */
    public JarFile(String name) {
	mName = new String(name);
	mIsUnpacked = false;
	mDir = null;
	mEJBNames = null;
    }


    /**
     * Extract all files of the jar file
     *
     * @param     dir path of the directory where the jar file has to be unpacked
     *                ("" or "." for the current working directory)
     * @exception Exception In error case
     */
    /*
     * Walk through the entry of the jar file,
     * If the entry is a directory, create this directory,
     * Else, create the directory parent, and copy the entry content in a file
     */
    public void unpack(String dir) throws Exception {

	if (dir.equals("")) {
	    mDir = new String("."+File.separator);
	} else {
	    mDir = new String(dir+File.separator);
	}

	ZipInputStream z = null;
	try {
	    z = new ZipInputStream(new FileInputStream(mName));
	    ZipEntry ze = z.getNextEntry();
	    while (ze != null) {
		File f = new File(mDir+ze.getName());
		if (ze.isDirectory()) {
		    if (!f.exists()) {
			if (!f.mkdirs()) {
			    throw new Exception("Cannot mkdir " + f.getPath());
			}
		    }
		} else {
		    if (f.getParent() != null) {
			File d = new File(f.getParent());
			if (! d.exists()) {
			    if (! d.mkdirs()) {
				throw new Exception("Cannot mkdir " + f.getPath());
			    }
			}
		    }
		    FileOutputStream fos = new FileOutputStream(mDir+ze.getName());
		    int i;
		    while ((i=z.read()) != -1) {
			fos.write(i);
		    }
		    fos.close();
		}
		ze = z.getNextEntry();
	    }
	} catch (Exception e) {
	    Trace.errln(e, false);
	    throw new Exception("Cannot unpack " + mName + " in " + mDir);
	} finally {
	    if (z != null) {
		try {
		    z.close();
		} catch (IOException e) {
		    Trace.errln(e, false);
		    throw new Exception("Cannot close the jar file " + mName);
		}
	    }
	}
	mIsUnpacked = true;
    }


    /**
     * Is the jar file an EJB-jar file ?
     * <br>
     * Note: The method unpack() has to be call before.
     *
     * @result    true if it is an EJB-jar file, false otherwise
     * @exception Exception In error case
     */


    public boolean isEJBJarFile() throws Exception {
	/*
	 * WARNING: To accept the ejb-jar files given by Sun for the DeployThon for
	 *          the JavaOne conference (not compliance ejb-jar files !!),
	 *          the analyse of the manifest file to get the names of the
	 *          deployment descriptors files have changed.
	 *
	 * Analyse the manifest file
	 * - search the line:
	 *     Enterprise-Bean: True
	 *   to check if it is an ejb-jar file
	 * - search the lines:
	 *     Name: ........
	 *   and retain the names with a '.ser' suffix.
	 */
	if (!mIsUnpacked) {
	    throw new Exception("The jar file '" + mName + "' must have been unpacked before");
	}
	boolean isEJB = false;
	BufferedReader mf = null;
	try {
	    mf = new BufferedReader(new FileReader(mDir+MANIFEST_NAME));
	    mEJBNames = new Vector();
	    String line;
	    line = mf.readLine();
	    while (line != null) {
		String Name = null;
		while ((line != null) && (line.trim().length() != 0)) {
		    StringTokenizer lst = new StringTokenizer(line, ":", false);
		    if (lst.countTokens() >= 2) {
			String stag = lst.nextToken().trim();
			if (stag.equals("Name")) {
			    Name = lst.nextToken().trim();
			}
			if (stag.equals("Enterprise-Bean")) {
			    if (lst.nextToken().trim().equals("True")) {
				isEJB = true;
			    }
			}
		    }
		    line = mf.readLine();
		}
		if (Name != null) {
		    if (Name.endsWith(".ser")) {
			mEJBNames.addElement(mDir+Name);
		    }
		}
		while ((line != null) && (line.trim().length() == 0)) {
		    line = mf.readLine();
		}
	    }
	} catch (Exception e) {
	    Trace.errln(e, false);
	    throw new Exception("Cannot analyse the manifest of the jar file " + mName +
				" in " + mDir);
	}

	return(isEJB);
    }

    // Old Version
      /*
       * Analyse the manifest file
       * Search the lines:
       *   Name: ........
       *   Enterprise-Bean: True
       */
    /*
      public boolean isEJBJarFile() throws Exception {
      if (!mIsUnpacked) {
      throw new Exception("The jar file '" + mName + "' must have been unpacked before");
      }
      BufferedReader mf = null;
      try {
      mf = new BufferedReader(new FileReader(mDir+MANIFEST_NAME));
      mEJBNames = new Vector();
      String line;
      line = mf.readLine();
      while (line != null) {
      String Name = null; boolean isEJB = false;
      while ((line != null) && (line.trim().length() != 0)) {
      StringTokenizer lst = new StringTokenizer(line, ":", false);
      if (lst.countTokens() >= 2) {
      String stag = lst.nextToken().trim();
      if (stag.equals("Name")) {
      Name = lst.nextToken().trim();
      }
      if (stag.equals("Enterprise-Bean")) {
      if (lst.nextToken().trim().equals("True")) {
      isEJB = true;
      }
      }
      }
      line = mf.readLine();
      }
      if ((Name != null) && (isEJB)) {
      mEJBNames.addElement(mDir+Name);
      }
      while ((line != null) && (line.trim().length() == 0)) {
      line = mf.readLine();
      }
      }
      } catch (Exception e) {
      Trace.errln(e, false);
      throw new Exception("Cannot analyse the manifest of the jar file " + mName +
      " in " + mDir);
      }

      return(mEJBNames.size()!=0);
      }
    */

    /**
     * Return the list of the EJB Names, 
     * i.e. paths of the unpacked enterprise Bean's serialized
     *      deployment descriptors files.
     * <br>
     * Note: The method unpack() has to be call before.
     * @result    the list of the EJB Names, (null if the jar file is not an EJB-Jar file)
     * @exception Exception In error case
     */
    public String[] getEJBNames() throws Exception {
	if (isEJBJarFile()) {
	    String[] lst = new String[mEJBNames.size()];
	    for (int i = 0; i < mEJBNames.size(); i++) {
		lst[i] = new String((String)mEJBNames.elementAt(i));
	    }
	    return(lst);
	} else {
	    return(null);
	}
    }

}


