/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.common;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;

/**
 * This class manages reading of the properties used by EJB Server. 
 * This class adopts the singleton design-pattern. No need to instanciate it, 
 *  this is done by the private constructor called in the static reference.
 * This class provides as well a public constructor, to read other 
 *  properties files.
 */
public class Prop {

    static private Properties systEnv   = System.getProperties();
    static private String installRoot   = systEnv.getProperty("install.root");
    static private String fileSeparator = systEnv.getProperty("file.separator");
    static private String homeUser      = systEnv.getProperty("user.home");
    static public  Prop   svr           = new Prop();

    private Properties filesEnv       = null;
    private Properties allEnv         = null;
    private String     propFileName   = null;
    private boolean    initProp       = false;
    private String[]   propFilesNames = new String[3];
    
    /**
     * Private ctor to initialize static
     *
     */
    private Prop() {
	propFileName = "jonas.properties";
	readPropFiles(propFileName);
    }


    /**
     * Here are the rules to retrieve the properties.
     * Properties are first read in file getProperty("install.root")/fileName
     *  then overridden with values found in "$HOME/fileName" and then overridden 
     *  with values found in file "fileName".
     * These properties may be overridden again by system properties, given on the java 
     *  command line.
     */
    private void readPropFiles(String fileName) {

	initProp = true;

	if (installRoot == null) {
	    System.err.println("JOnAS configuration error!");
	    System.err.println("    System property \"install.root\" not set. Exiting ...");
	    System.exit(2);
	}

	installRoot = installRoot.trim();

	filesEnv = new Properties();

	// getProperty("install.root")/fileName
	propFilesNames[0] = installRoot + fileSeparator + fileName;
	// $HOME/fileName
	propFilesNames[1] = homeUser + fileSeparator + fileName;
	// fileName
	propFilesNames[2] = fileName;

	for (int i=0; i<propFilesNames.length; i++) {
	    try {
		FileInputStream is = new FileInputStream(propFilesNames[i]);
		filesEnv.load(is);
	    } catch (FileNotFoundException e) {
		if ((i==0) && (fileName.equals("jonas.properties"))) { 
		    System.err.println("JOnAS configuration error");
		    // we cant work without the install.root property file
		    System.err.println("    "+e);
		    System.exit(2);
		}
	    } catch (IOException e) {
		if ((i==0) && (fileName.equals("jonas.properties"))) { 
		    // we cant work without the install.root property file
		    System.err.println("    "+e);
		    System.exit(2);
		} else
		    System.err.println("    "+e);
	    }
	}

	allEnv = (Properties) filesEnv.clone(); 
	// Overriddes with syst properties
	if (fileName.equals("jonas.properties")) {
	    for (Enumeration e = systEnv.keys(); e.hasMoreElements();) {
		Object key   = e.nextElement();
		String value = ((String)systEnv.get(key)).trim();
		allEnv.put(key, (Object)value);
	    }
	}
    }


    /**
     * Displays the JOnAS properties values, as they are set by the 
     *  different property files.
     *
     */
    static public void main(String args[]) {
	for (Enumeration e = svr.filesEnv.keys(); e.hasMoreElements();) {
	    Object key   = e.nextElement();
	    Object value = svr.filesEnv.get(key);
	    System.out.println(key.toString() + "=" + value.toString());
	}	
    }


    /**
     * Returns properties file name.
     *
     * @return JOnAS properties file name
     */
    public String getPropFile() {
	return propFileName;
    }


    /**
     * Returns JOnAS environment as configured with files properties and
     *  system properties.
     * @return JOnAS properties
     */
    public Properties getEnv() {
	return allEnv;
    }


    /**
     * Returns JOnAS environment as configured with files properties only.
     *
     * @return JOnAS properties
     */
    public Properties getFilesEnv() {
	return filesEnv;
    }


    /**
     * Returns the value of the related property. Without default values.
     *
     * @return property value
     */
    public String getValue(String key) {
	String retProperty = null;
	if (initProp) {
	    retProperty = allEnv.getProperty(key);
	} 
	if (retProperty == null) {
	    System.err.println("JOnAS configuration error.");
	    System.err.println("    Property \""+key+"\" has not been set in any of the following properties files:");
	    for (int i=0; i<propFilesNames.length; i++) {
		if (i==0) 
		    System.err.print("    First read      : ");
		else 
		    System.err.print("    Overridden with : ");
		System.err.println(propFilesNames[i]);
	    }
	    System.err.println("    Please set the \""+key+"\" property in one of these files.");
	    System.exit(2);
	}
	return retProperty.trim();
    }
    

    /**
     * Returns the value of the related property. With default values.
     *
     * @return property value
     */
    public String getValue(String key, String defaultVal) {
	
	String retProperty = null;
	if (initProp) {
	    retProperty = allEnv.getProperty(key, defaultVal);
	} else return defaultVal;
	return retProperty.trim();
    }


    /**
     * Public ctor to initialize and read other properties files.
     * These other instances do not touch the static Prop svr. 
     */
    public Prop(String fileName) {
	readPropFiles(fileName);
    }
}
