/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.common;

import java.util.Hashtable;


class TLEntry {
    Object value;

    TLEntry(Object object) {
        value = object;
    }
}

/**
 * This class provides ThreadLocal variables. These variables differ from their
 * normal counterparts in that each thread that accesses one (via its get or set
 * method) has its own, independently initialized copy of the variable.
 * ThreadLocal objects are typically private static variables in classes
 * that wish to associate state with a thread (e.g., a user ID or Transaction ID). 
 *
 * Each thread holds an implicit reference to its copy of a ThreadLocal as long as
 * the thread is alive and the ThreadLocal object is accessible; after a thread goes
 * away, all of its copies of ThreadLocal variables are subject to garbage collection
 * (unless other references to these copies exist). 
 *
 * This class has been written from the specifications of ThreadData in JDK 1.2.
 * Will be obsolate as soon as ThreadLocal is provided in JDK.
 */
public synchronized class ThreadLocal {

    // -------------------------------------------------------------------
    // private data
    // -------------------------------------------------------------------

    Hashtable htable;

    // -------------------------------------------------------------------
    // constructors
    // -------------------------------------------------------------------

    /**
     * Creates a ThreadLocal variable.
     */
    public ThreadLocal() {
        htable = new Hashtable();
    }

    // -------------------------------------------------------------------
    // public interface
    // -------------------------------------------------------------------

    /**
     * Sets the calling thread's instance of this ThreadLocal variable to the
     * given value. This is only used to change the value from the one assigned
     * by the initialValue method, and many applications will have no need for
     * this functionality.
     *
     * @param value - the value to be stored for the calling threads
     */
    public void set(Object object) {

	// retrieve entry for the current thread
        Thread thread = Thread.currentThread();
        TLEntry tle = (TLEntry) htable.get(thread);
        if (tle == null) {
            tle = new TLEntry(null);
            htable.put(thread, tle);
        }

	// set its value
        tle.value = object;
    }

    /**
     * Returns the value in the calling thread's copy of this ThreadLocal
     * variable. Creates and initializes the copy if this is the first time
     * the thread has called this method.
     */
    public Object get() {

	// retrieve entry for the current thread
        Thread thread = Thread.currentThread();
        TLEntry tle = (TLEntry) htable.get(thread);
        if (tle == null) {
            tle = new TLEntry(initialValue());
            htable.put(thread, tle);
        }

	// get its value
        return tle.value;
    }

    /**
     * Returns the calling thread's initial value for this ThreadLocal variable. 
     * This method will be called once per accessing thread for each ThreadLocal, 
     * the first time each thread accesses the variable with get or set.
     * If the programmer desires ThreadLocal variables to be initialized to some
     * value other than null, ThreadLocal must be subclassed, and this method
     * overridden. Typically, an anonymous inner class will be used. Typical
     * implementations of initialValue will call an appropriate constructor and 
     * return the newly constructed object.
     */
    protected Object initialValue() {
        return null;
    }

}
