/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.container;

import java.lang.reflect.Method;
import java.lang.reflect.Field;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Properties;
import javax.ejb.EJBHome;
import javax.ejb.EJBMetaData;
import javax.ejb.EJBObject;
import javax.ejb.Handle;
import javax.ejb.HomeHandle;
import javax.ejb.RemoveException;
import javax.ejb.deployment.AccessControlEntry;
import javax.ejb.deployment.ControlDescriptor;
import javax.ejb.deployment.DeploymentDescriptor;
import javax.ejb.deployment.SessionDescriptor;
import javax.transaction.Transaction;
import javax.transaction.TransactionManager;
import org.objectweb.jonas.common.Env;
import org.objectweb.jonas.common.Prop;
import org.objectweb.jonas.common.Trace;


/**
 * This class implements javax.ejb.EJBHome interface.
 * This class must be extended by the EJBHome class generated for a given bean.
 * 
 * Contains all information related to the bean.
 */
public abstract class JBeanHome extends UnicastRemoteObject implements EJBHome  {

    private static final int trace = Trace.DB_16;

    /**
     * @serial
     */
    private DeploymentDescriptor ejbDescriptor = null; 

    /**
     * @serial
     */
    private JBeanMetaData ejbMetaData = null; 

    /**
     * @serial
     */
    private String beanName = null;

    /**
     * @serial
     */
    private Properties ejbEnv = null;

    /**
     * @serial
     */
    private HomeHandle handleEJBHome = null;

    /**
     * @serial
     */
    private boolean isSession = true;

    /**
     * @serial
     */
    private boolean TxBeanManaged = false;


    /**
     * @serial
     */
    TransactionManager tm = Container.current;

    // ------------------------------------------------------------------
    // Constructors
    // ------------------------------------------------------------------

    /**
     * Constructor for the base class of the specific generated Home object.
     */
    public JBeanHome(DeploymentDescriptor desc, String beanName) throws RemoteException {

	Trace.outln(trace, "JBeanHome constructor for "+beanName);

	this.beanName   = beanName;
	ejbDescriptor   = desc;

	// Adds bean properties to the JOnAS Server properties
	Properties env  = desc.getEnvironmentProperties();
	try { 
	    // We must clone the result or all home will share the same environment!
	    ejbEnv = (Properties) Prop.svr.getEnv().clone();
	    for (Enumeration e = env.keys(); e.hasMoreElements() ;) {
		Object key   = e.nextElement();
		Object value = env.get(key);
		ejbEnv.put(key, value);
	    }
	} catch (Exception e) {
	    Trace.errln("JBeanHome ctor: "+e);
	}
	methodDescriptorsInit();
	register();
    }

    // ------------------------------------------------------------------
    // EJBHome implementation
    // ------------------------------------------------------------------

    /**
     * Obtains the EJBMetaData interface for the enterprise Bean. 
     * @return The enterprise Bean's EJBMetaData
     */
    public EJBMetaData getEJBMetaData() throws RemoteException {
	Trace.outln(trace, "JBeanHome.getEJBMetaData()");
	return ejbMetaData;
    }

    /**
     * Removes an EJB object identified by its handle. 
     */
    abstract public void remove(Handle handle) throws RemoteException, RemoveException;

    /**
     * Removes an EJB object identified by its primary key.
     */
    abstract public void remove(Object primaryKey) throws RemoteException, RemoveException;

    /**
     * Obtain a handle for the home object. The handle can be used at later time to re-obtain a
     * reference to the home object, possibly in a different Java Virtual Machine.
     * 
     * @return A handle for the home object.
     * 
     * @exception java.rmi.RemoteException - Thrown when the method failed due to a system-level
     * failure.
     */
    public HomeHandle getHomeHandle() throws java.rmi.RemoteException {

	Trace.outln(trace, "JBeanHome.getHomeHandle()");

	if (handleEJBHome == null) {
	    handleEJBHome = new HomeHandleImpl(getDeploymentDescriptor().getBeanHomeName().toString(), Env.getSyntaxJNDI());
	}
	return handleEJBHome;
    }

    // ------------------------------------------------------------------
    // Other methods
    // ------------------------------------------------------------------

    /**
     * register this bean to JNDI (rebind)
     */
    public void register() {
	String name = null;
	try {
	    name = ejbDescriptor.getBeanHomeName().toString();
	    Trace.outln (trace, "JBeanHome.register() "+ name);
	    Env.getInitialContext().rebind(name, this);
	} catch (Exception e) {
	    Trace.errln("Cannot register "+name+" : " + e);
	    System.exit(1);
	}
    }

    /**
     * unregister this bean in JNDI (unbind)
     */
    public void unregister() {
	String name = null;
	try {
	    name = ejbDescriptor.getBeanHomeName().toString();
	    Trace.outln (trace, "JBeanHome.unregister() "+ name);
	    Env.getInitialContext().unbind(name);
	} catch (Exception e) {
	    Trace.errln("Cannot unregister "+name+" : " + e);
	    System.exit(1);
	}
    }

    public void setEJBMetaData(JBeanMetaData md) {
	Trace.outln (trace, "JBeanHome.setEJBMetaData() ");	
	ejbMetaData = md;
    }

    /**
     * Is used only to see if a bean is TX_BEAN_MANAGED
     */
    private void methodDescriptorsInit() {
	Trace.outln (trace, "JBeanHome.methodDescriptorsInit()" );

	ControlDescriptor[] cd = ejbDescriptor.getControlDescriptors();
	Trace.outln(trace, "methodDescriptorsInit: ControlDescriptor[] size = " + cd.length);

	// Check that TX_BEAN_MANAGED transaction attribute are not mixed
	// Maybe we should raise an exception when mixed attributes are found ?
	boolean isContainerManaged = false;
	TxBeanManaged = false;
	for (int i = 0; i < cd.length; i++) {
	    if (cd[i].getTransactionAttribute() ==  ControlDescriptor.TX_BEAN_MANAGED) {
		TxBeanManaged = true;
		if (isContainerManaged) {
		    Trace.errln("WARNING : TX_BEAN_MANAGED transaction attribute must not be mixed");
		}
	    } else {
		isContainerManaged = true;
		if (TxBeanManaged) {
		    Trace.errln("WARNING : TX_BEAN_MANAGED transaction attribute must not be mixed");
		}
	    }
	}
    }


    /**
     * return true if the bean manages itself the transactions
     */
    public boolean isTxBeanManaged() {
	return TxBeanManaged;
    }

    // DeploymentDescriptor
    public DeploymentDescriptor getDeploymentDescriptor() {
	Trace.outln (trace, "JBeanHome.getDeploymentDescriptor() ");
	return ejbDescriptor;
    }


    /**
     * Returns the environment associated with the Bean
     */
    public Properties getEnvironment() {
	Trace.outln (trace, "JBeanHome.getEnvironment() for "+beanName);
	return ejbEnv;
    }


    /**
     * A possible method for 'cleaning' the state of the Bean after an unchecked exception
     *  thrown by an enterprise Bean's method (as described in EJB_Specifications & 12.3.3)
     *  (Not yet implemented)
     *
     * @param ue the unchecked exception thrown by an enterprise Bean's method
     * @exception RemoteException: Thrown when the method failed due to a system-level failure. 
     *
     */
    public void onUncheckedException (Throwable ue) throws java.rmi.RemoteException {
	Trace.outln(trace, "JBeanHome.onUncheckedException(Throwable "+ue.toString()+")");
	// TO BE DONE
	return;
    }


    /**
     * Gets the full name of the enterprise Bean's ClassName
     */
    public String getEnterpriseBeanClassName(){
	return ejbDescriptor.getEnterpriseBeanClassName();
    }
}
