/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.container; 
 
import java.util.Hashtable;
import java.util.Stack;
import java.util.EmptyStackException;
import java.rmi.RemoteException;
import javax.ejb.EJBObject;
import javax.ejb.EntityBean;
import javax.ejb.FinderException;
import javax.ejb.deployment.EntityDescriptor;
import javax.ejb.RemoveException;
import javax.ejb.Handle;
import javax.naming.Context;
import javax.sql.DataSource;
import javax.transaction.Transaction;
import org.objectweb.jonas.common.Env;
import org.objectweb.jonas.common.Trace;


/**
 * Class inherited by all the Home object for entity beans
 */
abstract public class JEntityHome extends JBeanHome {
    
    private final static int trace = Trace.DB_16;

    /**
     * @serial
     */
    private Hashtable ejbobjects = new Hashtable();

    /**
     * @serial
     */
    private Object datasource = null;

    /**
     * @serial
     */
    private EntityBean available  = null;

    /**
     * @serial
     */
    protected String primaryKeyClassName = null;

    /**
     * constructor
     * The DataSourceImpl param value may be null
     */
    public JEntityHome(EntityDescriptor desc, String beanName, Object ds) 
	throws RemoteException {

	super(desc, beanName);

	Trace.outln(trace, "JEntityHome constructor for "+beanName);

	this.datasource = ds;
	this.primaryKeyClassName = desc.getPrimaryKeyClassName();

	JBeanMetaData bbdm = new JBeanMetaData(this,
						     desc.getHomeInterfaceClassName(),
						     desc.getRemoteInterfaceClassName(),
						     primaryKeyClassName,
						     false,
						     false);
	this.setEJBMetaData(bbdm);
    }


    /**
     * Maps the  specified primary key to the EJBObject (only id the key does not exist)
     *
     * @param pk - the primary key 
     * @param ejbObject - the EJBObject 
     *
     */
    public synchronized void putEJBObject(Object pk, JBeanEntity ejbObject) {

	Trace.outln (trace, "JEntityHome.putEJBObject()");

	String s = pk.toString();

	// The key already exists -> return.
	if (ejbobjects.containsKey(s))
	    return;

	ejbobjects.put(s, ejbObject);
	Trace.outln (trace, "JEntityHome.putEJBObject() " +pk+ " is put in EJBObject table");

	try {
	    Transaction tx = tm.getTransaction();
	    if (tx != null) {
		HomeSynchro hs = new HomeSynchro(this, pk);
		// register this object as a synchronization
		tx.registerSynchronization(hs);
		Trace.outln(trace, "JEntityHome.putEJBObject() registerSynchronization("+this+")");
	    }
	} catch (Exception e) {
	    Trace.errln("JEntityHome.putEJBObject() cannot register synchronization Object: "+e);
	}
    }


    /**
     * Returns the EJBObject to which the specified primary key is mapped
     *
     * @param  pk - the primary key 
     * @return the value to which the key is mapped in this hashtable; 
     *         null if the key is not mapped to any value in this hashtable. 
     *
     */
    public JBeanEntity getEJBObject(Object pk) {

	Trace.outln (trace, "JEntityHome.getEJBObject()"+pk.toString());

	JBeanEntity bb = (JBeanEntity) ejbobjects.get(pk.toString());
	if (bb == null) {
	    Trace.outln (trace, "There is no EJBObject for this primary key "+pk);
	}
	// Do not check isMarkedRemoved() anymore: not really useful and it
	// leads to memory leaks
	return bb;
    }
  

    /**
     * 
     */
    public synchronized void removeEJBObject(Object pk) {

	Trace.outln (trace, "JEntityHome.removeEJBObject() "+pk.toString());

	Object obj = ejbobjects.remove(pk.toString());
    }


    /**
     * Returns the DataSource associated to the bean
     *
     * @return  the DataSource associated to the bean or null if there is no associated DataSource
     *
     */
    public Object getDataSource() throws RemoteException {
	Trace.outln (trace, "JEntityHome.getDataSource()");
	if (datasource == null) {
	    Trace.errln("JOnAS configuration error!");
	    Trace.errln("    JEntityHome.getDataSource() returning null");
	}
	return datasource;
    }


    /**
     * Get a pseudo instance of the bean for finder methods
     * This instance is never destroyed and can be reused.
     */
    public EntityBean getAvailable() throws RemoteException {

	if (available != null) {
	    Trace.outln(trace, "JEntityHome.getAvailable (found)");
	    return available;
	}
	Trace.outln(trace, "JEntityHome.getAvailable (not found)");

	String cn = null;
	Class c = null;
	try {
	    cn = getEnterpriseBeanClassName();
	    c = Class.forName(cn);
	} catch (ClassNotFoundException e) {
	    Trace.errln("createInstance: Failed to find class "+cn);
	    throw new RemoteException(" Container fails to find class "+cn, e);
	}

	try {
	    available = (EntityBean)c.newInstance();
	} catch (Exception e) {
	    Trace.errln("createInstance: "+ cn+" Fails to create a newInstance "+ e);
	    throw new RemoteException("Container cannot create a new instance of "+cn, e);
	}

	EntityContextImpl ctx = new EntityContextImpl(this);
	available.setEntityContext(ctx);

	return available;
    }

    /**
     * Removes an EJB object identified by its primary key.
     */
    public void remove(Object primaryKey) throws RemoteException, RemoveException {

	Trace.outln (trace, "JEntityHome.remove(pk)");

	JBeanEntity bb = getEJBObject(primaryKey);
	if (bb == null) {
	    throw new RemoveException("ObjectNotFound");
	} else {
	    bb.remove();
	}
    }

    /**
     * Removes an EJB object identified by its handle. 
     */
    public void remove(Handle handle) throws RemoteException, RemoveException {

	Trace.outln(trace,"remove(Handle handle)");

	EJBObject ejbobj = handle.getEJBObject();
	ejbobj.remove();
    }
}
