/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.container;

import java.rmi.RemoteException;
import javax.ejb.SessionBean;
import javax.ejb.SessionSynchronization;
import javax.ejb.deployment.ControlDescriptor;
import javax.transaction.Synchronization;
import javax.transaction.SystemException;
import javax.transaction.Status;
import org.objectweb.jonas.common.Trace;

/**
 * This object is build for every StateFul SessionBean instance, even if
 * it does not implement SessionSynchronization. This class implements the
 * JTA Synchronization interface.
 */
public class SessionSynchroImpl extends SynchroImpl {
    
    SessionSynchronization instance = null; // the bean instance to synchronize

    /** 
     * Constructor on SessionSynchroImpl is called when the bean is created, that
     *  is when create() is invoked on Home interface.
     * SessionSynchroImpl object and Bean object are associated.
     * 
     */
    public SessionSynchroImpl(SessionSynchronization bean) {
	Trace.outln(trace, "SessionSynchroImpl.SessionSynchroImpl(SessionBean bean)");
	reset();
	instance = bean;
     }
 
    // -------------------------------------------------------------------
    // Synchronization implementation
    // -------------------------------------------------------------------

   /** 
     * This beforeCompletion method is called by the transaction 
     * manager prior to the start of the transaction completion process.
     * This method executes in the transaction context of the calling 
     * thread.
     */
    public void beforeCompletion() {

	Trace.outln(trace, "SessionSynchroImpl.beforeCompletion()");

	if (instance != null) {
	    try {
		instance.beforeCompletion();
	    } catch (RemoteException e) {
		Trace.errln("beforeCompletion exception:"+e);
	    }
	}
    }


    /** 
     * The afterCompletion method is called by the transaction 
     * manager after the transaction is committed or rolled back. 
     * This method executes without a transaction context.
     *
     * @param status The status of the transaction completion.
     */
    public void afterCompletion(int status) {

	Trace.outln(trace, "SessionSynchroImpl.afterCompletion(Status "+status+") ");

	if (instance != null) {
	    try {
		instance.afterCompletion(status == Status.STATUS_COMMITTED);
	    } catch (RemoteException e) {
		Trace.errln("afterCompletion exception:"+e);
	    }
	}
	reset();		// even if not a SessionSynchronization.
    }


    // -------------------------------------------------------------------
    // Other methods
    // -------------------------------------------------------------------

    /**
     * This method considers the incoming bean request, and manages the 
     *  synchronization jobs to be done for the bean. 
     * Nothing is to be done outside a transaction scope.
     * If the bean was not yet involved in a transaction, registers the 
     *  synchro object in the TM, and calls the bean afterBegin() method.
     * If the bean was already involved in a transaction, and a request 
     *  arrives for an other transaction, then the thread is suspended, 
     *  otherwise the request is taken into account.
     */
    public void considerRequest() throws RemoteException {
	Trace.outln(trace, "SessionSynchroImpl.considerRequest() ");

	// No transaction => no synchronization.
	try {
	    currentCoord = tm.getTransaction();
	} catch (SystemException e) {
	    Trace.outln(Trace.DB_31, "considerRequest: Cannot get transaction");
	}
	if (currentCoord == null) {
	    Trace.outln(trace, "SessionSynchroImpl.considerRequest() currentCoord == null");
	    // A Session Bean cannot be called in different transactions
	    if (beanCoord != null) {
		Trace.errln("The Session Bean cannot be called in a different context");
		throw new RemoteException("The Session Bean cannot be called in a different context");
	    }
	    return;
	}

	// If this is not the first request for this Tx, return.
	// Note that if the request is called outside any Tx, it's like
	// if each request were in a separate Tx.
	if (beanCoord != null) {
	    // A Session Bean cannot be called in different transactions
	    if (beanCoord.equals(currentCoord) == false) {
		Trace.outln(Trace.DB_31, "Bean already engaged in an other transaction");
		throw new RemoteException("The Session Bean cannot be called in a different context");
	    }		
	    return;
	}

	// just to remember that we are participating in a transaction.
	beanCoord = currentCoord;
	
	// register this object as a synchronization
	try {
	    currentCoord.registerSynchronization(this);
	} catch (Exception e) {
	    Trace.errln("SessionSynchroImpl.considerRequest() registerSynchronization(this) : "+e);
	    return;
	}

	// Calls afterBegin on SessionSynchro interface implemented by the bean
	if (instance != null) {
	    try {
		instance.afterBegin();
	    } catch (Exception e) {
		Trace.errln("SessionSynchroImpl.considerRequest() afterBegin() : "+e);
	    } 
	}
    }

    public boolean inTransaction() {
	if (beanCoord != null) {
	    Trace.outln(trace, "SessionBean participating in a transaction.");
	    return true;
	} else {
	    Trace.outln(trace, "SessionBean not involved in a transaction.");
	    return false;
	}
    }
}
