/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.jdbc_xa;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.Vector;
import javax.transaction.xa.XAResource;
import javax.sql.ConnectionEvent;
import javax.sql.ConnectionEventListener;
import javax.sql.XAConnection;
import org.objectweb.jonas.common.Trace;

/**
 * This is the implementation of the XAConnection interface. This object
 * encapsulates 2 objects: the Connection and the XAResource.
 * Connection is used for std SQL primitive, XAResource is used for
 * XA primitive (transaction demarcation).
 */
class XAConnectionImpl implements XAConnection {

    private static final int trace = Trace.DB_10;

    XAResource xaRes = null;
    ConnectionImpl myConn = null;
    Connection actConn = null;
    Vector eventListeners = new Vector();

    // -----------------------------------------------------------------
    // Constructors
    // -----------------------------------------------------------------

    public XAConnectionImpl(Connection conn, XADataSourceImpl ds) {

	Trace.outln(trace, "XAConnectionImpl.XAConnectionImpl(Connection conn, XADataSourceImpl ds)");

	this.actConn = conn;

	// An XAConnection holds 2 objects: 1 Connection + 1 XAResource
	this.myConn = new ConnectionImpl(this, conn);
	this.xaRes = new XAResourceImpl(this, conn, ds);
    }

    // -----------------------------------------------------------------
    // XAConnection implementation
    // -----------------------------------------------------------------

    /**
     * Return an XA resource to the caller.
     *
     * @return The XAResource
     *
     * @exception SQLException - if a database-access error occurs
     */
    public XAResource getXAResource() throws SQLException {

	Trace.outln(trace, "XAConnectionImpl.getXAResource()");

	return xaRes;
    }

    // -----------------------------------------------------------------
    // PooledConnection implementation
    // -----------------------------------------------------------------

    /**
     * Create an object handle for a database connection.
     *
     * @exception SQLException - if a database-access error occurs
     */
    public Connection getConnection() throws SQLException {

	Trace.outln(trace, "XAConnectionImpl.getConnection()");

	// Just return the already created object.
	return myConn;
    }

    /**
     * Close the database connection.
     *
     * @exception SQLException - if a database-access error occurs
     */
    public void close() throws SQLException {

	Trace.outln(trace, "XAConnectionImpl.close()");

	// Close the actual Connection here.
	actConn.close();
	actConn = null;

	// We can forget Connection and XAResource too.
	xaRes = null;
	myConn = null;
    }

    /**
     * Add an event listener.
     *
     * @param listener event listener
     */
    public void addConnectionEventListener(ConnectionEventListener listener) {

	Trace.outln(trace, "XAConnectionImpl.addConnectionEventListener()");

	eventListeners.addElement(listener);
    }

    /**
     * Remove an event listener.
     *
     * @param listener event listener
     */
    public void removeConnectionEventListener(ConnectionEventListener listener) {

	Trace.outln(trace, "XAConnectionImpl.removeConnectionEventListener()");

	eventListeners.removeElement(listener);
    }

    // -----------------------------------------------------------------
    // Other methods
    // -----------------------------------------------------------------

    /*
     * Notify a Close event on Connection
     */
    public void notifyClose() {

	Trace.outln(trace, "XAConnectionImpl.notifyClose()");

	// Notify event to listeners
	for (int i = 0; i < eventListeners.size(); i++) {
	    ConnectionEventListener l = (ConnectionEventListener) eventListeners.elementAt(i);
	    l.connectionClosed(new ConnectionEvent(this));
	}
    }

    /*
     * Notify an Error event on Connection
     */
    public void notifyError(SQLException ex) {

	Trace.outln(trace, "XAConnectionImpl.notifyError("+ex+")");

	// Notify event to listeners
	for (int i = 0; i < eventListeners.size(); i++) {
	    ConnectionEventListener l = (ConnectionEventListener) eventListeners.elementAt(i);
	    l.connectionErrorOccurred(new ConnectionEvent(this, ex));
	}
    }


}
