/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.jdbc_xa;

import java.io.PrintWriter;
import java.io.Serializable;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import javax.sql.XAConnection;
import javax.sql.XADataSource;
import javax.transaction.xa.XAResource;
import org.objectweb.jonas.common.Trace;

/**
 * The XADataSource implementation in our pseudo JDBC-XA driver.
 * Acts as a factory for XAConnection objects.
 * Created by our DataSource implementation, in the EJBServer.
 * There is one XADataSource for each DataSource JDBC which do not have
 * this kind of driver in standard (all databases today!)
 */
public class XADataSourceImpl implements XADataSource {

    private static final int trace = Trace.DB_10;

    // properties
    private String dataSourceName = null;
    private String url = null;
    private String className = null;
    private String userName = null;
    private String password = null;

    // private
    private int loginTimeout = 60;
    private PrintWriter log = null;
    private Class driverClass = null;

    // -----------------------------------------------------------------
    // Constructors
    // -----------------------------------------------------------------

    /**
     * Constructor for Factory
     */
    public XADataSourceImpl() {
	Trace.outln(trace, "XADataSourceImpl.XADataSourceImpl()");
    }

    // -----------------------------------------------------------------
    // Properties
    // -----------------------------------------------------------------
    public String getDataSourceName() {
	return dataSourceName;
    }
    public void setDataSourceName(String s) {
	Trace.outln(trace, "XADataSourceImpl.setDataSourceName("+s+")");
	dataSourceName = s;
    }

    public String getUrl() {
	return url;
    }
    public void setUrl(String s) {
	url = s;
    }

    public String getClassName() {
	return className;
    }
    public void setClassName(String s) {
	className = s;

	// Loads standard JDBC driver and keeps it loaded (via driverClass)
	Trace.outln(trace, "Load JDBC driver "+s);
	try {
	    driverClass = Class.forName(className);
	} catch (java.lang.ClassNotFoundException e) {
	    Trace.errln("Cannot load JDBC driver : " + e);
	}
    }

    public String getUserName() {
	return userName;
    }
    public void setUserName(String s) {
	userName = s;
    }

    public String getPassword() {
	return password;
    }
    public void setPassword(String s) {
	password = s;
    }


    // -----------------------------------------------------------------
    // XADataSource implementation
    // -----------------------------------------------------------------

    /**
     * Attempt to establish a database connection.
     *
     * @return a Connection to the database
     *
     * @exception SQLException  if a database-access error occurs.
     */
    public XAConnection getXAConnection() throws SQLException {
	return getXAConnection(userName, password);
    }


    /**
     * Attempt to establish a database connection.
     *
     * @param user The database user on whose behalf the Connection is being made
     *
     * @param passwd the user's password
     *
     * @return a Connection to the database
     *
     * @exception SQLException  if a database-access error occurs.
     */
    public XAConnection getXAConnection(String user, String passwd) throws SQLException {

	Trace.outln(trace, "XADataSourceImpl.getXAConnection(String "+user+", String "+passwd+") on "+url);

	// Create the actual connection in the std driver
	Connection conn = null;
	try {
	    if (user.length() == 0 || passwd.length() == 0) {
		conn = DriverManager.getConnection(url);
		Trace.outln(trace, "    * New Connection on "+url);
	    } else {
		conn = DriverManager.getConnection(url, user, passwd);
		Trace.outln(trace, "    * New Connection on "+url+ " for "+user);
	    }
	} catch (SQLException e) {
	    Trace.errln("Could not get Connection on "+url+" for "+user+"/"+passwd);
	    throw e;
	}

	// Create the XAConnection object
	XAConnectionImpl xac = new XAConnectionImpl(conn, this);

	// return the XAConnection
	return (XAConnection) xac;
    }

    /**
     * Get the log writer for this data source. 
     *
     * @return PrintWritert he log writer for this data source, null if disabled
     *
     * @exception SQLException  if a database-access error occurs.
     */
    public PrintWriter getLogWriter() throws SQLException {
	return log;
    }

    /**
     * Set the log writer for this data source. 
     *
     * @param out the new log writer; to disable, set to null
     *
     * @exception SQLException  if a database-access error occurs.
     */
    public void setLogWriter(PrintWriter out) throws SQLException {
	log = out;
    }


    /**
     * Sets the maximum time in seconds that this data source will wait while attempting 
     * to connect to a database.
     *
     * @param seconds the data source login time limit
     *
     * @exception SQLException  if a database-access error occurs.
     */
    public void setLoginTimeout(int seconds) throws SQLException {
	loginTimeout = seconds;
    }

    /**
     * Gets the maximum time in seconds that this data source can wait 
     * while attempting to connect to a database.
     *
     * @return the data source login time limit
     *
     * @exception SQLException  if a database-access error occurs.
     */
    public int getLoginTimeout() throws SQLException {
	return loginTimeout;
    }

}
