/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.jtm;

import java.util.Vector;
import java.rmi.RemoteException;
import org.objectweb.jonas.common.Trace;

/**
 * Timer associated to a transaction
 *
 * @see ControlImpl
 */
public class Timer {

    /**
     * Constructor
     *
     * @param ctrl		Transaction using this timer
     * @param timeout		Timeout in seconds for this transaction
     * @param act		Action when the timer expires
     */
    public Timer(ControlImpl ctrl, long timeout, int act) {
	
	Trace.outln(trace, "Timer constructor: "+timeout+", "+act);

	control = ctrl;
	reminding = timeout;
	action = act;
	synchronized (timerList) {
	    timerList.addElement(this);
	}
    }

    /**
     * Change timeout and/or action associated to this timer
     *  - since this timer is in the list, we must synchronize this
     *    with the monitor of the list.
     */
    void change(long timeout, int act) {
	
	Trace.outln(trace, "Timer change: "+timeout+", "+act);

	synchronized (timerList) {
	    reminding = timeout;
	    action = act;
	}
    }
	
    /**
     * Invalidate this timer
     */
    void invalidate() {
	
	Trace.outln(trace, "Timer invalidate");

	synchronized (timerList) {
	    reminding = BIG_TIME;
	    action = NO_ACTION;
	}
    }
	
    /**
     * Get reminding time before timer expires
     */
    long getReminding() {
	if (reminding <= 0 || action != ROLLBACK)
	    return 0;
	else
	    return reminding;
    }

    /**
     * Get action associated with this timer
     */
    int getAction() {
	return action;
    }

    /**
     * Update timer every second. Used by clock.
     * - this must be called with the timerList monitor.
     */
    long update() {
	return --reminding;
    }

    /**
     * Process the timer
     */
    int process() {
	switch (action) {

	case ROLLBACK:
	    Trace.outln(trace, "Timer: timeout expired, rollback transaction");
	    try {
		// try to rollback transaction
		control.ding();
		action = DELETE;
		reminding = TIME_TO_DELETE;
		return 1; // keep it
	    } catch (InactiveException e) {
		Trace.errln("Timer: transaction inactive");
	    } catch (Exception e) {
		Trace.errln("Timer: rollback caught exception "+e);
	    }
	    break;

	case DELETE:
	    Trace.outln(trace, "Timer: Delete Transaction object");
	    // nothing to do (garbage collector will remove control)
	    break;

	case NO_ACTION:
	    reminding = BIG_TIME;
	    return 1;	// keep it

	default:
	    Trace.errln("Timer: Bad value for action: "+action);
	    break;
	}
	return 0;		// remove it
    }

    /**
     * update all timers in the list
     * each timer expired is put in a special list of expired timers
     * they will be processed then by the Batch Thread.
     */
    static void updateAll() {	
	while (true) {
	    try {
		Thread.currentThread().sleep(1000);	// 1 second
		synchronized (timerList) {
		    int found = 0;
		    for (int i = 0; i < timerList.size(); i++) {
			Timer t = (Timer) timerList.elementAt(i);
			if (t.update() <= 0) {
			    timerList.removeElementAt(i--);
			    expiredList.addElement(t);
			    found++;
			}
		    }
		    if (found > 0) {
			timerList.notify();
		    }
		}
	    } catch (InterruptedException e) {
		Trace.errln("Timer interrupted");
	    }
	}
    }

    /**
     * process all expired timers
     */
    static void processAll() {
	while (true) {
	    Timer t;
	    synchronized (timerList) {
		while (expiredList.isEmpty()) {
		    try {
			timerList.wait();
		    } catch (Exception e) {
			Trace.errln("Exception in Batch: "+e);
		    }
		}
		t = (Timer) expiredList.elementAt(0);
		expiredList.removeElementAt(0);
	    }
	    // Do not keep the lock during the processing of the timer
	    if (t.process() != 0) {
		// put back timer in the timerlist
		synchronized (timerList) {
		    timerList.addElement(t);
		}
	    }
	}
    }

    /**
     * Search in the timerlist if a transaction matching this propagation
     * context exists.
     *
     * @return		The ControlImpl object, or null if not found.
     */
    static ControlImpl getControl(PropagationContext ctx) {
	Trace.outln(trace, "Timer.getControl");

	ControlImpl ret = null;

	synchronized (timerList) {
	    for (int i = 0; i < timerList.size(); i++) {
		Timer t = (Timer) timerList.elementAt(i);
		try {
		    Coordinator coord = t.control.get_coordinator();
		    if (coord.equals(ctx.getCoordinator())) {
			Trace.outln(trace, "Timer.getControl: Control already in the list");
			ret = t.control;
			break;
		    }
		} catch (RemoteException e) {
		    Trace.errln("Timer.getControl: remote exception:\n" + e);
		}
	    }
	}
	return ret;
    }

    /**
     * Action associated to this timer: NO_ACTION, ROLLBACK or DELETE.
     */
    public static final int NO_ACTION = 0;
    public static final int ROLLBACK = 1;
    public static final int DELETE = 2;
    private int action = NO_ACTION;

    private static final long TIME_TO_DELETE = 30;
    private static final long BIG_TIME = 55555;

    private ControlImpl control;
    private long reminding;

    // static lists
    private static Vector timerList = new Vector();
    private static Vector expiredList = new Vector();

    private static int trace = Trace.DB_4;
}
