/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.lib;


import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectInputStream;
import java.lang.String;
import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.StringTokenizer;
import javax.ejb.deployment.ControlDescriptor;
import javax.ejb.deployment.DeploymentDescriptor;
import javax.ejb.deployment.EntityDescriptor;
import javax.ejb.deployment.SessionDescriptor;
import org.objectweb.jonas.common.Trace;


/**
 * Represents
 * <ul>
 * <li>
 * a javax.ejb.deployment.EntityDescriptor or
 * <li>
 * a javax.ejb.deployment.SessionDescriptor,
 * </ul>
 * and implements some tool methods.
 * @version    %I%, %G%
 */


public class DD {

    private EntityDescriptor mED;
    private SessionDescriptor mSD;

    /**
     * Construct a DD from an Enterprise Bean serialized deployment descriptor.
     * @param fileName path of the Enterprise Bean serialized deployment descriptor file
     * @exception IOException In error case
     */
    public DD(String fileName) throws IOException {
	mED = null; mSD = null;
	try {
	    ObjectInput in1 = new ObjectInputStream (new FileInputStream(fileName)); 
	    mSD = (SessionDescriptor)in1.readObject();
	} catch (ClassCastException e) {
	    ObjectInput in2 = new ObjectInputStream (new FileInputStream(fileName));
	    try {
		mED = (EntityDescriptor)in2.readObject();
	    } catch (ClassCastException cce) {
		Trace.errln(cce, false);
		throw new IOException("'"+fileName+
				      "' is not a serialized EntityDescriptor or SessionDescriptor");
	    } catch (ClassNotFoundException cnfe) {
		Trace.errln(cnfe, false);
		throw new IOException("Cannot read the serialized DeploymentDescriptor file '"
				      +fileName+"'");
	    }
	} catch (ClassNotFoundException e) {
	    Trace.errln(e, false);
	    throw new IOException("Cannot read the serialized DeploymentDescriptor file '"
				  +fileName+"'");
	}
    }

    /**
     * Returns true if it is an EntityDescriptor.
     */
    public boolean isEntity() {
	return(mED != null);
    }

    /**
     * Returns true if it is an SessionDescriptor.
     */
    public boolean isSession() {
	return(mSD != null);
    }

    /**
     * Returns the DeploymentDescriptor object, (null in error case).
     */
    public DeploymentDescriptor getDeploymentDescriptor() {
	if (mSD != null) {
	    return((DeploymentDescriptor)mSD);
	} else if (mED != null) {
	    return((DeploymentDescriptor)mED);
	} else {
	    return(null);
	}
    }


    /**
     * Returns the EntityDescriptor object, (null if it is not an EntityDescriptor).
     */
    public EntityDescriptor getEntityDescriptor() {
	return(mED);
    }

    /**
     * Returns the SessionDescriptor object, (null if it is not an SessionDescriptor).
     */
    public SessionDescriptor getSessionDescriptor() {
	return(mSD);
    }

    /**
     * Returns the Primary Key class of an EntityDescriptor.
     * @exception Exception in error case
     */
    public Class getPrimaryKey() throws Exception {
	if (mED == null) {
	    throw new Exception("No primary key for a session descriptor");
	}

	try {
	    String pkName;
	    Class  pkClass;
	    pkName = mED.getPrimaryKeyClassName();
	    pkClass = Class.forName(pkName);
	    return(pkClass);
	} catch (Exception e) {
	    Trace.errln(e, false);
	    throw new Exception("Cannot get the fields of the Primary Key of the EntityDescriptor");
	}
    }

    /**
     * Returns the Fields of the Primary Key class of an EntityDescriptor.
     * @exception Exception in error case
     */
    public Field[] getFieldsOfPrimaryKey() throws Exception {
	if (mED == null) {
	    throw new Exception("No primary key for a session descriptor");
	}
	Class  pkClass = getPrimaryKey();
	return(pkClass.getDeclaredFields());
    }

    /**
     * Returns true if it is an EntityDescriptor describing an Entity Bean with
     * container-managed persistence type.
     */
    // See EJB-Specification V1.0, & 9.10.1, page 75
    public boolean isEntityContainerManagedPersistence() {
	boolean val = false;
	if (mED != null) {
	    val = (mED.getContainerManagedFields() != null);
	}
	return(val);
    }

    /**
     * Returns the ControlDescriptor associated to the given method of the bean.
     * @param method a method of the bean class or of the remote interface of the bean
     * @return the associated ControlDescriptor
     * @exception Exception In case of there no bean level ControlDescriptor
     *                      or if there is no ControlDescriptor for the method.
     * Note: The given method is not strictly compared with the methods of ControlDescriptor
     *       Only the names, the return type and the paramaters lists are compared.
     */
    // See EJB_Specification V1.0, & 11.7, page 105
    public ControlDescriptor getControlDescriptorForMethod(Method method) throws Exception {
	ControlDescriptor val = null;
	DeploymentDescriptor dd = null;
	if (mSD != null) {
	    dd = (DeploymentDescriptor)mSD;
	} else if (mED != null) {
	    dd = (DeploymentDescriptor)mED;
	}
	ControlDescriptor cdBean = null; // ControlDescriptor Bean-level
	ControlDescriptor cdMeth = null; // ControlDescriptor Method-level
	ControlDescriptor [] cds = dd.getControlDescriptors();
	if (cds == null) {
	    throw new Exception("No ControlDescriptor for the method '"+method.toString()+"'");
	}
	for (int i=0; (i<cds.length) && (cdMeth==null); i++) {
	    Method m = cds[i].getMethod();
	    if (m == null) {
		cdBean = cds[i];
	    } else if (methodsEquals(m, method)) {
		cdMeth = cds[i];
	    }
	}
	if (cdMeth != null) {
	    val = cdMeth;
	} else if (cdBean != null) {
	    val = cdBean;
	} else {
	    throw new Exception("No ControlDescriptor for the method '"+method.toString()+"'");
	}
	return(val);
    }

    /**
     * Returns the transaction attribute defined in the ControlDescriptor
     * associated to the given method of the bean.
     * @param method a method of the bean class or of the remote interface of the bean
     * @return the associated transaction attribute
     * @exception Exception In case of there no bean level ControlDescriptor
     *                      or if there is no ControlDescriptor for the method.
     * Note: The given method is not strictly compared with the methods of ControlDescriptor
     *       Only the names, the return type and the paramaters lists are compared.
     */
    public int getTransactionAttributeForMethod(Method method) throws Exception {
	ControlDescriptor cd = getControlDescriptorForMethod(method);
	return(cd.getTransactionAttribute());
    }

    /**
     * Returns true if the two given methods are similars.
     * Compare the names, the return type and the parameter types.
     * (The modifiers of the methods and the throw clauses are not compared.)
     */
    public static boolean methodsEquals(Method m1, Method m2) {
	// Compare the names, then the return type and at last the parameter types
	return(methodsEquals(m1, m2, m1.getName(), m2.getName()));
    }

    /**
     * Returns true if the two given methods are similars.
     * Compare the given names, the return type and the parameter types.
     * (The modifiers of the methods and the throw clauses are not compared.)
     * The compared names of the methods are the given parameters names.
     */
    public static boolean methodsEquals(Method m1, Method m2,
					String m1Name, String m2Name) {
	return(methodsEquals(m1, m2, m1Name, m2Name, true));
    }

    /**
     * Returns true if the two given methods are similars.
     * Compare the given names, the return type if it is asked, and the parameter types.
     * (The modifiers of the methods and the throw clauses are not compared.)
     * The return types are compared if it is asked.
     * The compared names of the methods are the given parameters names. 
     */
    public static boolean methodsEquals(Method m1, Method m2,
					String m1Name, String m2Name,
					boolean compareReturn) {
	// Compare the given names,
	// then the return type of the methods,
	// and at last the parameter types of the methods
	if (! m1Name.equals(m2Name)) {
	    return(false);
	}
	if (compareReturn) {
	    if (! (m1.getReturnType()).equals(m1.getReturnType())) {
		return(false);
	    }
	}
	Class[] p1 = m1.getParameterTypes();
	Class[] p2 = m2.getParameterTypes();
	if (p1.length != p2.length) {
	    return(false);
	}
	for (int i=0; i<p1.length; i++) {
	    if (! p1[i].equals(p2[i])) {
		return(false);
	    }
	}
	return(true);
    }

    /**
     * For a container-managed persistence Entity,
     * returns the name of the database table in which the bean infos are stored.
     * (db.TableName property in the ContainerProperties file)
     * @return    database table name
     * @exception Exception if the db.TableName property is not defined
     * in the EnvironmentProperties.
     */
    public String getDBTableName() throws Exception {
	String val = null;
	if (mED == null) {
	    throw new Exception("db.TableName property can be get only in an EntityDescriptor");
	}
	val = mED.getEnvironmentProperties().getProperty("db.TableName");
	if (val == null) {
	    throw new Exception("'db.TableName' property not defined in the ContainerProperties");
	}
	return(val);
    }

    /**
     * For a container-managed persistence Entity,
     * returns the name of the database column associated to the given bean's field.
     * (db.Field.<field-name> property in the ContainerProperties file)
     * @return    database column name associated to the given field
     * @exception Exception if the db.Field.<field-name> property is not defined
     * in the EnvironmentProperties.
     */
    public String getDBFieldName(Field f) throws Exception {
	String val = null;
	if (mED == null) {
	    throw new Exception("db.Field.* properties can be get only in an EntityDescriptor");
	}
	val = mED.getEnvironmentProperties().getProperty("db.Field."+f.getName());
	if (val == null) {
	    throw new Exception("'db.Field."+f.getName()+"' "+
				"property not defined in the ContainerProperties");
	}
	return(val);
    }

    /**
     * For a container-managed persistence Entity,
     * returns the 'where' clause of the given finder method defined in the ContainerProperties.
     * (db.Finder.<method-name>.<param-number> property in the ContainerProperties file)
     * @return    'where' clause in a string format
     * @exception Exception if the db.Finder.<method-name> property is not defined
     * in the EnvironmentProperties.
     */
    public String getDBFinderWhereClause(Method m) throws Exception {
	String val = null;
	if (mED == null) {
	    throw new Exception("db.Finder.* properties can be get only in an EntityDescriptor");
	}
	val = mED.getEnvironmentProperties().getProperty("db.Finder."+m.getName());
	if (val == null) {
	    throw new Exception("'db.Finder."+m.getName()+"' "+
				"property not defined in the ContainerProperties");
	}
	return(val);
    }

    /**
     * For a container-managed persistence Entity,
     * returns the 'isModifiedMethodName' property defined in the ContainerProperties.
     * (isModifiedMethodName property in the ContainerProperties file)
     * @return    the string which is the property of the isModifiedMethodName,
     *            null if the property 'isModifiedMethodName' is not defined.
     */
    public String getIsModifiedMethodName() {
	String val = null;
	if (mED != null) {
	    val = mED.getEnvironmentProperties().getProperty("isModifiedMethodName");
	}
	return(val);
    }

}

