/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.tools;

import java.io.IOException;
import org.objectweb.jonas.common.JarFile;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas.lib.DD;
import org.objectweb.jonas.lib.InvalidBeanException;
import org.objectweb.jonas.lib.JavaType;


/**
 *  This class allows to check the validity of a given Enterprise Java Bean.
 */

public class CheckEJB {

    static final int trace  = Trace.DB_19;

    private String    ddName = null;
    private CheckBean cBean  = null;

    public CheckEJB(String fileSerName) throws InvalidBeanException, IOException {
	DD ddesc = null;

	// Get the DeploymentDescriptor
	ddesc  = new DD(fileSerName);
	ddName = new String(fileSerName);
	if (ddesc.isSession()) {
	    // Session
	    Trace.outln(trace, "CheckEJB: This is a descriptor of a Session Bean");
	    Trace.outln(trace, "  "+"'"+fileSerName+"'");
	    cBean  = new CheckSession(ddesc);
	} else {
	    // Entity
	    if (ddesc.isEntityContainerManagedPersistence()) {
		// Entity Persistence Container Managed
		Trace.outln(trace, "CheckEJB: This is a descriptor of an Entity Bean with");
		Trace.outln(trace, "          container-managed persistence");
		Trace.outln(trace, "  "+"'"+fileSerName+"'");
		cBean  = new CheckEntityCMbyJDBC(ddesc);
	    } else {
		// Entity Persistence Bean Managed
		Trace.outln(trace, "CheckEJB: This is a descriptor of an Entity Bean with");
		Trace.outln(trace, "          bean-managed persistence");
		Trace.outln(trace, "  "+"'"+fileSerName+"'");
		cBean  = new CheckEntityBM(ddesc);
	    }
	}
    }

    public boolean check() throws InvalidBeanException {
	boolean isValid = cBean.check();
	return(isValid);
    }

    /**
     * CheckEJB allows to verify the validity of
     * given Enterprise Java Bean(s).
     * <p>
     *
     * Usage: java org.objectweb.jonas.tools.CheckEJB -help
     * <br>
     *         to print this help message
     * <p>
     *
     * or     java org.objectweb.jonas.tools.CheckEJB <Options> <Input_File>
     * <br>
     *         to verify the validity of given Enterprise Java Bean(s).
     * <p>
     *           Options include:
     * <ul>
     * <li>
     * </ul>
     * <p>
     *           Input_File         file name of the serialized deployment descriptor
     *                              (.ser ended), or file name of the EJB-jar (.jar ended).
     */

    public static void main(String args[]) {

	boolean  isHelp        = false;
	boolean  isVerbose     = false;
	String   fileInputName = null;
	String[] fileSerNames  = null;
	int      exitValue     = 0;

	// Get args
	for (int argn=0; argn<args.length; argn++) {
	    String arg = args[argn];
	    if (arg.equals("-help") || arg.equals("-?")) {
		isHelp = true;
		continue;
	    }
	    if (arg.equals("-verbose")) {
		isVerbose = true;
		continue;
	    }
	    fileInputName = args[argn];
	}

	// Trace configure
	if (isVerbose) {
	    Trace.configure("out", "err", trace, false);
	} else {
	    Trace.configure("out", "err", Trace.DB_NONE, false);
	}

	// Usage ?
	if (isHelp) {
	    usage();
	    System.exit(0);
	}

	// Check args
	if (fileInputName == null) {
	    usage();
	    System.exit(2);
	}

	// Get the list of the EJBs to process
	if ((! fileInputName.endsWith(".jar")) && (! fileInputName.endsWith(".ser"))) {
	    Trace.errln("CheckEJB FATAL ERROR: File name not ended with '.jar' or '.ser'");
	    Trace.errln("  '" + fileInputName + "'");
	    Trace.errln("");
	    System.exit(2);
	} else if (fileInputName.endsWith(".jar")) {
	    try {
		JarFile jf = new JarFile(fileInputName);
		String dirOutputName = new String(".");
		jf.unpack(dirOutputName);
		fileSerNames = jf.getEJBNames();
		if (fileSerNames == null) {
		    Trace.errln("CheckEJB FATAL ERROR: There is no bean in " +fileInputName);
		    Trace.errln("");
		    System.exit(2);
		}
	    } catch (Exception e) {
		Trace.errln("CheckEJB FATAL ERROR: " + e.getMessage());
		Trace.errln("");
		System.exit(2);
	    }
	} else if (fileInputName.endsWith(".ser")) {
	    fileSerNames = new String[1];
	    fileSerNames[0] = new String(fileInputName);
	}

	// For each EJB to process, generates the EJB Home and the EJB Object classes
	for (int i=0; i<fileSerNames.length; i++) {
	    CheckEJB cejb;
	    try {
		cejb = new CheckEJB(fileSerNames[i]);
		if (cejb.check()) {
		    Trace.outln("CheckEJB: Valid Enterprise Java Bean");
		    Trace.outln("  '"+fileSerNames[i]+"'");
		} else {
		    Trace.errln("CheckEJB ERROR: Invalid Enterprise Java Bean");
		    Trace.errln("  '" + fileSerNames[i] + "'");
		    exitValue = 2;
		}
	    } catch (IOException e) {
		Trace.errln("CheckEJB FATAL ERROR: " + e);
		Trace.errln("  '" + fileSerNames[i]+"'");
		exitValue = 2;
	    } catch (InvalidBeanException e) {
		Trace.errln("CheckEJB FATAL ERROR: " + e.getMessage());
		Trace.errln("CheckEJB ERROR: Invalid Enterprise Java Bean");
		Trace.errln("  '" + fileSerNames[i] + "'");
		exitValue = 2;
	    }
	}
	// End
	Trace.errln("");
	System.exit(exitValue);
    }

    private static void usage() {
	Trace.outln("");
	Trace.outln("Usage: java org.objectweb.jonas.tools.CheckEJB -help");
	Trace.outln("         to print this help message");
	Trace.outln("");
	Trace.outln("");
	Trace.outln(" or    java org.objectweb.jonas.tools.CheckEJB <Options> <Input_File>");
	Trace.outln("");
	Trace.outln("         to verify the validity of the given EJB(s).");
	Trace.outln("");
	Trace.outln("           Options include:");
	Trace.outln("             -verbose");
	Trace.outln("");
	Trace.outln("           Input_File         file name of the serialized deployment descriptor");
	Trace.outln("                              (.ser ended), or");
	Trace.outln("                              file name of the EJB-jar (.jar ended).");
	Trace.outln("");
    }

}

