/*
 * The contents of this file are subject to the JOnAS Public License Version 
 * 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License on the JOnAS web site.
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.
 * See the License for the specific terms governing rights and limitations under 
 * the License.
 *
 * The Original Code is JOnAS application server code released July 1999.
 *
 * The Initial Developer of the Original Code is Bull S.A.
 * The Original Code and portions created by Bull S.A. are
 *    Copyright (C) 1999 Bull S.A. All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * --------------------------------------------------------------------------
 * $Id$
 * --------------------------------------------------------------------------
 */


package org.objectweb.jonas.tools;


import java.io.IOException;
import java.lang.String;
import javax.ejb.deployment.DeploymentDescriptor;
import org.objectweb.jonas.common.IndentedFile;
import org.objectweb.jonas.common.Trace;
import org.objectweb.jonas.lib.BeanNaming;
import org.objectweb.jonas.lib.DD;


/**
 *  This class allows to generate the source of the class that implements
 *  the Handle of a given Enterprise Java Bean.
 *  (Only useful for Entity Bean)
 */

abstract class GenICHandle {

    static final int trace = Trace.DB_19;

    IndentedFile src   = null;

    DD           ddesc = null;

    // Full names of the interfaces and the bean classe
    String ejbRemoteName = null;
    String ejbHomeName   = null;
    String ejbBeanName   = null;
    // Classes of the interfaces and the bean
    Class  ejbRemoteClass = null;
    Class  ejbHomeClass   = null;
    Class  ejbBeanClass   = null;
    // Names of the implementation classes (Full names and base names)
    String wrpRemoteName     = null;
    String wrpHomeName       = null;
    String wrpHandleName     = null;
    String wrpRemoteBaseName = null;
    String wrpHomeBaseName   = null;
    String wrpHandleBaseName = null;

    /**
     * GenICHandle Constructor
     *
     * @param dd       deployment descriptor of the Enterprise Java Bean
     * @param fileName name of the source file of the class to generate
     * @exception GenICException in error case
     */
    GenICHandle(DD dd, String fileName) throws GenICException {
	// Source File
	try {
	    src = new IndentedFile(fileName);
	} catch (IOException e) {
	    Trace.errln(e, true);
	    throw new GenICException("Cannot create '" + fileName + "' file");
	}
	// Deployment Descriptor
	ddesc = dd;
	// Names building
	DeploymentDescriptor deploymentDesc = ddesc.getDeploymentDescriptor();
	ejbRemoteName = deploymentDesc.getRemoteInterfaceClassName();
	if ((ejbRemoteName == null) || (ejbRemoteName.length() < 1)) {
	    throw new GenICException("The RemoteInterfaceClassName MUST BE defined "+
				     "in the DeploymentDescriptor");
	}
	ejbHomeName = deploymentDesc.getHomeInterfaceClassName();
	if ((ejbHomeName == null) || (ejbHomeName.length() < 1)) {
	    throw new GenICException("The HomeInterfaceClassName MUST BE defined "+
				     "in the DeploymentDescriptor");
	}
	ejbBeanName = deploymentDesc.getEnterpriseBeanClassName();
	if ((ejbBeanName == null) || (ejbBeanName.length() < 1)) {
	    throw new GenICException("The EnterpriseBeanClassName MUST BE defined "+
				     "in the DeploymentDescriptor");
	}
	BeanNaming bn     = new BeanNaming(ddesc.getDeploymentDescriptor());
	wrpRemoteBaseName = bn.getWrpRemoteName();
	wrpRemoteName     = bn.getFullWrpRemoteName();
	wrpHomeBaseName   = bn.getWrpHomeName();
	wrpHomeName       = bn.getFullWrpHomeName();
	wrpHandleBaseName = bn.getWrpHandleName();
	wrpHandleName     = bn.getFullWrpHandleName();

    }

    /**
     * Generate the source of the class which implements the handle
     * @param withError the source file is generated with a syntax error
     *                  (internal use only for test)
     * @exception GenICException in error case
     */
    void generate(boolean withError) throws GenICException {

	// Comments
	src.println("//");
	src.println("// "+wrpHandleBaseName+".java");
	src.println("// DO NOT EDIT THIS FILE");
	src.println("// It is automatically generated via the GenIC tool");
	src.println("//");
	src.println();

	// Package clause
	String s = BeanNaming.getPackageName(wrpHandleName);
	if (s.length() != 0) {
	    src.println("package "+s+";");
	}
	src.println();

	// Wrong import clause generation for test only !!!!!
	if (withError) {
	    src.println("import java.toto.Titi;");
	    src.println();
	}

	// Import clauses
	genImport();
	src.println();

	// Class declaration
	src.println("public class "+wrpHandleBaseName+
		    " extends " + getSuperClass() + " {");
	src.println();
	src.indentPlus();

	// Attributes declaration
	genAttributes();
	src.println();

	// Constructor()
	genConstructor();
	src.println();

	// getEJBObject() Methode
	genGetEJBObject();
	src.println();

	// Class End
	src.indentMinus();
	src.println("}");
	src.println();

	//
	src.close();
    }

    /**
     * Name of the superclass of the generated class
     */
    abstract String getSuperClass();

    /**
     * Source generation of the import clauses
     * @exception GenICException in error case
     */
    abstract void   genImport() throws GenICException;

    /**
     * Source generation of the attributes part
     * @exception GenICException in error case
     */
    abstract void   genAttributes() throws GenICException;

    /**
     * Source generation of the constructor method
     * @exception GenICException in error case
     */
    abstract void   genConstructor() throws GenICException;

    /**
     * Source generation of the getEJBObject() method
     * @exception GenICException in error case
     */
    abstract void   genGetEJBObject() throws GenICException;

}

